/*****************************************************************************\
 * DateCalendars.h
 *
 * DateCalendars contains calendar calculations
 *
 * author: mark huss (mark@mhuss.com)
 * Based on Bill Gray's open-source code at projectpluto.com
 *
\*****************************************************************************/

// uncomment the following line to include an assortment of non-western
// calendars:
// #define CALENDARS_OF_THE_WORLD

#if !defined( DATE_CALENDARS__H )
#define DATE_CALENDARS__H

typedef int MonthDays[13];

typedef long YearEndDays[2];

enum CalendarType {
  T_GREGORIAN = 0,
  T_JULIAN = 1
#if defined(CALENDARS_OF_THE_WORLD)
  ,T_HEBREW = 2,
  T_ISLAMIC = 3,
  T_REVOLUTIONARY = 4,
  T_PERSIAN = 5
#endif
};

class DateCalendars {

public:

    static int getCalendarYear(long jd, CalendarType calendar);

    static void getJulGregYearData(
        int year, long& days, MonthDays& md, bool julian );

    static int getCalendarData(
        int year, YearEndDays& days, MonthDays& md, CalendarType calendar );


private:

    enum CalendarEpoch {
        E_JULIAN_GREGORIAN = 1721060L
  #if defined(CALENDARS_OF_THE_WORLD)
        ,E_ISLAMIC = 1948086L,
        E_HEBREW = 347996L,
        E_REVOLUTIONARY = 2375475L
  #endif
    };

  // ---------------------------------------------------------------------------
  #if defined( CALENDARS_OF_THE_WORLD )

    enum {
        HALAKIM_IN_DAY = (24L * 1080L),
        JALALI_ZERO = 1947954L,
        LOWER_PERSIAN_YEAR = -1096,
        UPPER_PERSIAN_YEAR = 2327
    };

    static int mod( int x, int y );

    // Islamic calendar
    static void getIslamicYearData( int year, int& days, MonthDays& md );

    // Hebrew calendar
    static int lunationsToTishri1( int year );
    static void lunationsToDaysAndHalakim(
        int lunations, int& days, int& halakim);
    static void findTishri1( int year, int& days, int& halakim );
    static void getHebrewYearData( int year, YearEndDays& days, MonthDays& md );

    // French Revolutionary calendar
    static int jdOfFrenchRevYear( int year );
    static void getRevolutionaryYearData(
        int year, YearEndDays& days, MonthDays& md );

    // Persian (Jalaali) calendar
    static int jalaliJd0( int jalaliYear );
    static void getJalaliYearData(
        const int year, YearEndDays& days, MonthDays& md );

  #endif  /* #if defined( CALENDARS_OF_THE_WORLD ) */
  // ---------------------------------------------------------------------------

};



#endif  /* #if !defined( DATE_CALENDARS__H ) */

