// LunarTerms.h - a list of terms needed to do lunar calcs

// author: mark huss (mark@mhuss.com)

// Two terms corrected 2013-05-26 (thanks to Mattias Holm)

#if !defined( LUNAR_TERMS__H )
#define LUNAR_TERMS__H

#include "Lunar.h"

// terms for longitude & radius
//
struct LunarLonRadTerm {
  int  d,   // multiple of mean elongation
       m,   // multiple of mean anomaly of the sun
       mp,  // multiple of mean anomaly of the moon
       f;   // multiple of mean distance of the Moon from its ascending node
  long sl,  // Coefficient of the sine of the argument
       sr;  // Coefficient of the cosine of the argument
};

// terms for latitude
//
struct LunarLatTerm {
  int  d,   // multiple of mean elongation
       m,   // multiple of mean anomaly of the sun
       mp,  // multiple of mean anomaly of the moon
       f;   // multiple of mean distance of the Moon from its ascending node
  long sb;  // Coefficient of the sine of the argument
};

// terms for phase
//
struct LunarTermsPh {
  int    e,   // earth orbit eccentricty
         m,   // sun's mean anomaly
         mp,  // moon's mean anomaly
         f,   // mean distance of the Moon from its ascending node
         om;  // Longitude of the ascending node of the lunar orbit
  double nm,  // new moon corrections
         fm;  // full moon corrections
};

// Lunar Fundimentals

static const int N_LFTERM = 5;

// moon's mean longitude
const double LunarFundimentals_Lp[] = {
       218.3164591,
    481267.88134236,
        -0.0013268,
         1.85583502e-006,
        -1.53494298e-008
};

// moon's mean elongation
const double LunarFundimentals_D[] = {
       297.8502042,
    445267.1115168,
        -0.00163,
         1.83194472e-006,
        -8.84447e-009
};

// sun's mean anomaly
const double LunarFundimentals_M[] = {
       357.5291092,
     35999.0502909,
        -0.0001536,
         4.08329931e-008,
         0.0
};

// moon's mean anomaly
const double LunarFundimentals_Mp[] = {
       134.9634114,
    477198.8676313,
         0.008997,
         1.43474081e-005,
        -6.79717238e-008
};

// mean distance of the Moon from its ascending node
const double LunarFundimentals_F[] = {
        93.2720993,
    483202.0175273,
        -0.0034029,
        -2.83607487e-007,
         1.15833246e-009
};

// Lunar Fundimentals fo Phase Calculation

static const int N_LUNAR_FUND_PH = 5;

static const double PhaseFundimentals_M[] = {
        2.5534,
       29.10535670,
       -0.0000014,
       -0.000000110,
        0.0
};

static const double PhaseFundimentals_Mp[] = {
      201.5643,
      385.81693528,
        0.0107582,
        0.00001238,
       -0.000000058
};

static const double PhaseFundimentals_F[] = {
      160.7108,
      390.67050284,
       -0.0016118,
       -0.00000227,
        0.000000011
};

static const double PhaseFundimentals_Om[] = {
      124.7746,
       -1.56375588,
        0.0020672,
        0.00000215,
        0.0
};

// Lunar phase terms

static const int N_LUNAR_TERMS_PH = 25;

static const LunarTermsPh LunarPhaseNF[] = {
//    E   M   Mp  F   Om  New       Full
    { 0,  0,  1,  0,  0, -0.40720, -0.40614 },
    { 1,  1,  0,  0,  0,  0.17241,  0.17302 },
    { 0,  0,  2,  0,  0,  0.01608,  0.01614 },
    { 0,  0,  0,  2,  0,  0.01039,  0.01043 },
    { 1, -1,  1,  0,  0,  0.00739,  0.00734 },
    { 1,  1,  1,  0,  0, -0.00514, -0.00515 },
    { 2,  2,  0,  0,  0,  0.00208,  0.00209 },
    { 0,  0,  1, -2,  0, -0.00111, -0.00111 },
    { 0,  0,  1,  2,  0, -0.00057, -0.00057 },
    { 1,  1,  2,  0,  0,  0.00056,  0.00056 },
    { 0,  0,  3,  0,  0, -0.00042, -0.00042 },
    { 1,  1,  0,  2,  0,  0.00042,  0.00042 },
    { 1,  1,  0, -2,  0,  0.00038,  0.00038 },
    { 1, -1,  2,  0,  0, -0.00024, -0.00024 },
    { 0,  0,  0,  0,  1, -0.00017, -0.00017 },
    { 0,  2,  1,  0,  0, -0.00007, -0.00007 },
    { 0,  0,  2, -2,  0,  0.00004,  0.00004 },
    { 0,  3,  0,  0,  0,  0.00004,  0.00004 },
    { 0,  1,  1, -2,  0,  0.00003,  0.00003 },
    { 0,  0,  2,  2,  0,  0.00003,  0.00003 },
    { 0,  1,  1,  2,  0, -0.00003, -0.00003 },
    { 0, -1,  1,  2,  0,  0.00003,  0.00003 },
    { 0, -1,  1, -2,  0, -0.00002, -0.00002 },
    { 0,  1,  3,  0,  0, -0.00002, -0.00002 },
    { 0,  0,  4,  0,  0,  0.00002,  0.00002 }
};

static const LunarTermsPh LunarPhaseQ[] = {
//    E   M   Mp  F   Om  1Q        3Q
    { 0,  0,  1,  0,  0, -0.62801, -0.62801 },
    { 1,  1,  0,  0,  0,  0.17172,  0.17172 },
    { 1,  1,  1,  0,  0, -0.01183, -0.01183 },
    { 0,  0,  2,  0,  0,  0.00862,  0.00862 },
    { 0,  0,  0,  2,  0,  0.00804,  0.00804 },
    { 1, -1,  1,  0,  0,  0.00454,  0.00454 },
    { 2,  2,  0,  0,  0,  0.00204,  0.00204 },
    { 0,  0,  1, -2,  0, -0.00180, -0.00180 },
    { 0,  0,  1,  2,  0, -0.00070, -0.00070 },
    { 0,  0,  3,  0,  0, -0.00040, -0.00040 },
    { 1, -1,  2,  0,  0, -0.00034, -0.00034 },
    { 1,  1,  0,  2,  0,  0.00032,  0.00032 },
    { 1,  1,  0, -2,  0,  0.00032,  0.00032 },
    { 2,  2,  1,  0,  0, -0.00028, -0.00028 },
    { 1,  1,  2,  0,  0,  0.00027,  0.00027 },
    { 0,  0,  0,  0,  1, -0.00017, -0.00017 },
    { 0, -1,  1, -2,  0, -0.00005, -0.00005 },
    { 0,  0,  2,  2,  0,  0.00004,  0.00004 },
    { 0,  1,  1,  2,  0, -0.00004,  0.00004 },
    { 0, -2,  1,  0,  0,  0.00004,  0.00004 },
    { 0,  1,  1, -2,  0,  0.00003,  0.00003 },
    { 0,  3,  0,  0,  0,  0.00003,  0.00003 },
    { 0,  0,  2, -2,  0,  0.00002,  0.00002 },
    { 0, -1,  1,  2,  0,  0.00002,  0.00002 },
    { 0,  1,  3,  0,  0, -0.00002, -0.00002 }
};

// Lunar longitude & radius terms

static const int N_LTERM = 60;

const LunarLonRadTerm LunarLonRad[] = {
//    D   M   M'  F   Sigma_l    Sigma_r
    { 0,  0,  1,  0,  6288774, -20905355 },
    { 2,  0, -1,  0,  1274027,  -3699111 },
    { 2,  0,  0,  0,   658314,  -2955968 },
    { 0,  0,  2,  0,   213618,   -569925 },
    { 0,  1,  0,  0,  -185116,     48888 },
    { 0,  0,  0,  2,  -114332,     -3149 },
    { 2,  0, -2,  0,    58793,    246158 },
    { 2, -1, -1,  0,    57066,   -152138 },
    { 2,  0,  1,  0,    53322,   -170733 },
    { 2, -1,  0,  0,    45758,   -204586 },
    { 0,  1, -1,  0,   -40923,   -129620 },
    { 1,  0,  0,  0,   -34720,    108743 },
    { 0,  1,  1,  0,   -30383,    104755 },
    { 2,  0,  0, -2,    15327,     10321 },
    { 0,  0,  1,  2,   -12528,         0 },
    { 0,  0,  1, -2,    10980,     79661 },
    { 4,  0, -1,  0,    10675,    -34782 },
    { 0,  0,  3,  0,    10034,    -23210 },
    { 4,  0, -2,  0,     8548,    -21636 },
    { 2,  1, -1,  0,    -7888,     24208 },
    { 2,  1,  0,  0,    -6766,     30824 },
    { 1,  0, -1,  0,    -5163,     -8379 },
    { 1,  1,  0,  0,     4987,    -16675 },
    { 2, -1,  1,  0,     4036,    -12831 },
    { 2,  0,  2,  0,     3994,    -10445 },
    { 4,  0,  0,  0,     3861,    -11650 },
    { 2,  0, -3,  0,     3665,     14403 },
    { 0,  1, -2,  0,    -2689,     -7003 },
    { 2,  0, -1,  2,    -2602,         0 },
    { 2, -1, -2,  0,     2390,     10056 },
    { 1,  0,  1,  0,    -2348,      6322 },
    { 2, -2,  0,  0,     2236,     -9884 },
    { 0,  1,  2,  0,    -2120,      5751 },
    { 0,  2,  0,  0,    -2069,         0 },
    { 2, -2, -1,  0,     2048,     -4950 },
    { 2,  0,  1, -2,    -1773,      4130 },
    { 2,  0,  0,  2,    -1595,         0 },
    { 4, -1, -1,  0,     1215,     -3958 },
    { 0,  0,  2,  2,    -1110,         0 },
    { 3,  0, -1,  0,     -892,      3258 },
    { 2,  1,  1,  0,     -810,      2616 },
    { 4, -1, -2,  0,      759,     -1897 },
    { 0,  2, -1,  0,     -713,     -2117 },
    { 2,  2, -1,  0,     -700,      2354 },
    { 2,  1, -2,  0,      691,         0 },
    { 2, -1,  0, -2,      596,         0 },
    { 4,  0,  1,  0,      549,     -1423 },
    { 0,  0,  4,  0,      537,     -1117 },
    { 4, -1,  0,  0,      520,     -1571 },
    { 1,  0, -2,  0,     -487,     -1739 },
    { 2,  1,  0, -2,     -399,         0 },
    { 0,  0,  2, -2,     -381,     -4421 },
    { 1,  1,  1,  0,      351,         0 },
    { 3,  0, -2,  0,     -340,         0 },
    { 4,  0, -3,  0,      330,         0 },
    { 2, -1,  2,  0,      327,         0 },
    { 0,  2,  1,  0,     -323,      1165 },
    { 1,  1, -1,  0,      299,         0 },
    { 2,  0,  3,  0,      294,         0 },
    { 2,  0, -1, -2,        0,      8752 }
};

// Lunar latitude terms

const LunarLatTerm LunarLat[] = {
//    D   M   M'  F   Sigma_b
    { 0,  0,  0,  1, 5128122 },
    { 0,  0,  1,  1,  280602 },
    { 0,  0,  1, -1,  277693 },
    { 2,  0,  0, -1,  173237 },
    { 2,  0, -1,  1,   55413 },
    { 2,  0, -1, -1,   46271 },
    { 2,  0,  0,  1,   32573 },
    { 0,  0,  2,  1,   17198 },
    { 2,  0,  1, -1,    9266 },
    { 0,  0,  2, -1,    8822 },
    { 2, -1,  0, -1,    8216 },
    { 2,  0, -2, -1,    4324 },
    { 2,  0,  1,  1,    4200 },
    { 2,  1,  0, -1,   -3359 },
    { 2, -1, -1,  1,    2463 },
    { 2, -1,  0,  1,    2211 },
    { 2, -1, -1, -1,    2065 },
    { 0,  1, -1, -1,   -1870 },
    { 4,  0, -1, -1,    1828 },
    { 0,  1,  0,  1,   -1794 },
    { 0,  0,  0,  3,   -1749 },
    { 0,  1, -1,  1,   -1565 },
    { 1,  0,  0,  1,   -1491 },
    { 0,  1,  1,  1,   -1475 },
    { 0,  1,  1, -1,   -1410 },
    { 0,  1,  0, -1,   -1344 },
    { 1,  0,  0, -1,   -1335 },
    { 0,  0,  3,  1,    1107 },
    { 4,  0,  0, -1,    1021 },
    { 4,  0, -1,  1,     833 },
    { 0,  0,  1, -3,     777 },
    { 4,  0, -2,  1,     671 },
    { 2,  0,  0, -3,     607 },
    { 2,  0,  2, -1,     596 },
    { 2, -1,  1, -1,     491 },
    { 2,  0, -2,  1,    -451 },
    { 0,  0,  3, -1,     439 },
    { 2,  0,  2,  1,     422 },
    { 2,  0, -3, -1,     421 },
    { 2,  1, -1,  1,    -366 },
    { 2,  1,  0,  1,    -351 },
    { 4,  0,  0,  1,     331 },
    { 2, -1,  1,  1,     315 },
    { 2, -2,  0, -1,     302 },
    { 0,  0,  1,  3,    -283 },
    { 2,  1,  1, -1,    -229 },
    { 1,  1,  0, -1,     223 },
    { 1,  1,  0,  1,     223 },
    { 0,  1, -2, -1,    -220 },
    { 2,  1, -1, -1,    -220 },
    { 1,  0,  1,  1,    -185 },
    { 2, -1, -2, -1,     181 },
    { 0,  1,  2,  1,    -177 },
    { 4,  0, -2, -1,     176 },
    { 4, -1, -1, -1,     166 },
    { 1,  0,  1, -1,    -164 },
    { 4,  0,  1, -1,     132 },
    { 1,  0, -1, -1,    -119 },
    { 4, -1,  0, -1,     115 },
    { 2, -2,  0,  1,     107 }
};

#endif  /* #if !defined( LUNAR_TERMS__H ) */
