/*****************************************************************************\
 * ObsInfo   holds latitude, longitude and TZ for an observing location
 *
 * author: mark huss (mark@mhuss.com)
 *
\*****************************************************************************/

#if !defined( OBS_INFO__H )
#define OBS_INFO__H

#include "Astro.h"
#include "MathOps.h"

// * * * * * Observer's Location Info * * * * *

class ObsInfo {
public:
  // c'tor: lon & lat are passed in DEGREES
  ObsInfo( double lon = 0, double lat = 0, int tz = 0 ) :
      m_longitude( MathOps::toRadians(lon) ),
      m_latitude( MathOps::toRadians(lat) ),
      m_timeZone( tz )
  {}

  double longitude() const { return m_longitude; }
  double degLatitude() const { return MathOps::toDegrees(m_latitude); }
  // set: lon passed in DEGREES
  void setLongitude(double lon) { m_longitude = MathOps::toRadians(lon); }

  double latitude() const { return m_latitude;  }
  double degLongitude() const { return MathOps::toDegrees(m_longitude); }
  // set: lat passed in DEGREES
  void setLatitude(double lat) { m_latitude = MathOps::toRadians(lat); }

  int timeZone() const { return m_timeZone;  }
  void setTimeZone(int tz) { m_timeZone = tz; }

private:
  double m_longitude;   // in radians, N positive
  double m_latitude;    // in radians, E positive
  int    m_timeZone;    // +/- 12 w.r.t. UTC
};

#endif  /* #if !defined( OBS_INFO__H ) */
