// utest.cpp

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include "ObsInfo.h"
#include "Lunar.h"
#include "AstroOps.h"
#include "DateOps.h"
#include "RiseSet.h"

ObsInfo oi(-75.12, 40.0, -5);

// ---------------------------------------------------------------------------
void printDate(double jd) {
  int day, month, year;
  double fullDay = floor(jd);
  DateOps::dayToDmy( long(fullDay), day, month, year );
  printf( "%02d/%02d/%04d ", month, day, year );
}

// ---------------------------------------------------------------------------
void printDateTime(double jd) {
  int day, month, year, hr, min, sec;

  DateOps::dayToDmy( long(floor(jd)), day, month, year );
  DateOps::dayToHMS( hr, min, sec, jd);
  printf( "%02d/%02d/%04d, %02d:%02d:%02d\n", month, day, year, hr, min, sec );
}

// ---------------------------------------------------------------------------
void printRS(double jd) {

  TimePairRS rs;
  RiseSet::getTimes( rs, RiseSet::MOON, jd, oi );
  double rise = rs.rise, set = rs.set;

  char riseTime[8] = { 0 };
  if ( rs.rise >= 0. )
    DateOps::formatTime(riseTime, rs.rise);
  else {
    TimePairRS rs0;
    RiseSet::getTimes( rs0, RiseSet::MOON, jd-1, oi );
    if ( rs0.rise >= 0. ) {
      rise = rs0.rise;
      DateOps::formatTime(riseTime, rs0.rise);
    }
  }

  char setTime[8] = { 0 };
  if ( rs.set >= 0. )
    DateOps::formatTime(setTime, rs.set);
  else {
    TimePairRS rs1;
    RiseSet::getTimes( rs1, RiseSet::MOON, jd+1, oi );
    if ( rs1.set >= 0. ) {
      set = rs1.set;
      DateOps::formatTime(setTime, rs1.set);
    }
  }

  if ( *riseTime ) {
    if ( *setTime ) {  // both
      if ( rise < set )
        printf( "Rises %s Sets %s\n", riseTime, setTime );
      else
        printf( "Sets %s Rises %s\n", setTime, riseTime );
    }
    else  // riseTime only
      printf( "Rises %s\n", riseTime);
  }
  else if ( *setTime ) // setTime only
    printf( "Sets %s\n", setTime );
}

// ****************************************************************************
int main(int argc, char** argv) {
  double jd = DateOps::dmyToDoubleDay(13, 10, 1992) - Astro::JD_DIFF;
  double E = AstroOps::equationOfTime(jd);
  double secs = (E - int(E)) * Astro::SECONDS_PER_MINUTE;
  printf( "jd=%f, E=%f\n", jd, E );
  printf( "%02d:%02.1f\n", int(E), secs );



/***
  double jd = AstroOps::equinoxSolstice( 1962, AstroOps::SUMMER );
  printf( "jd=%f\nShould be 06/21/1962  21:25:08\n       is ", jd );
  printDateTime(jd);

  for (int i=1996; i<2006; i++ ) {
    jd = AstroOps::equinoxSolstice( i, AstroOps::SPRING );
    printDateTime(jd);
    jd = AstroOps::equinoxSolstice( i, AstroOps::SUMMER );
    printDateTime(jd);
    jd = AstroOps::equinoxSolstice( i, AstroOps::AUTUMN );
    printDateTime(jd);
    jd = AstroOps::equinoxSolstice( i, AstroOps::WINTER );
    printDateTime(jd);
    puts("");
  }
 ***/


/***
  double jd1 = DateOps::dmyToDoubleDay( 31,10,2004, 3*Astro::SECONDS_PER_HOUR );
  printf( "jd1=%f, TZO=%f\n", jd1, DateOps::tzOffsetInDays(jd1) * 24.);
  printDateTime(jd1);

  time_t tt = DateOps::dayToTime(jd1);

  double jd2 = DateOps::timeToLDay(tt);
  printf( "jd2=%f\n", jd2);
  printDateTime(jd2);
 ***/

/***
  double jd = Lunar::nextMoon();
  printf( "Next Moon: " );
  printDate(jd);

  double ad = Lunar::getPhase( jd, Lunar::NEW );
  printf( "\nNM: " );
  printDateTime(ad);

  jd += 7;
  ad = Lunar::getPhase( jd, Lunar::Q1 );
  printf( "Q1: " );
  printDateTime(ad);

  jd += 7;
  ad = Lunar::getPhase( jd, Lunar::FULL );
  printf( "FM: " );
  printDateTime(ad);

  jd += 7;
  ad = Lunar::getPhase( jd, Lunar::Q3 );
  printf( "Q3: " );
  printDateTime(ad);
 ***/

/***
  double localOffset = -5.0/24.;

  double jd = DateOps::now(true);
  jd -= 1.5;
  for (int i = 0; i<12; i++) {
    double jd_ = floor(jd) - Astro::JD_DIFF - localOffset;
    printDate(jd);
    printRS(jd_);
    jd += .25;
  }

 ***/

/***
  double jd = DateOps::now(true);
  printDateTime(jd);
  puts("");

  jd -= .75;
  for (int i = 0; i<36; i++) {
    printDateTime(jd);
    jd += 1./24.;
  }
 ***/

/***
  long jDay = DateOps::dmyToDay( 14, 2, 1977);

  printf("\n\n*** Unit test phase calc using Meeus' example ***\n"
         "julianDay = %ld\n", jDay);

  double ad = Lunar::getPhase( jDay, Lunar::NEW, false );
  int day, month, year, h, m, s;

  DateOps::dayToDmy( ad, day, month, year );
  DateOps::toHMS( h, m, s, ad );

  printf("Ref  date/time: 1977-02-18 03:37:42\n"
         "Calc date/time: %04d-%02d-%02d %02d:%02d:%02d\n",
         year, month, day, h, m, s );

  for( int i=1; i<32; i++ ) {
    jDay = DateOps::dmyToDay( i, 1, 2004 );
    ad = Lunar::getPhase( jDay, Lunar::Q3 );
    printf( "%02d Q3: ", i );
    printDateTime(ad);
  }

  jDay = DateOps::dmyToDay( 14, 1, 2004 );
  ad = Lunar::getPhase( jDay, Lunar::Q3 );
  printf( "Q3: " );
  printDateTime(ad);

  jDay = DateOps::dmyToDay( 21, 1, 2004 );
  ad = Lunar::getPhase( jDay, Lunar::NEW );
  printf( "NM: " );
  printDateTime(ad);

  jDay = DateOps::dmyToDay( 28, 1, 2004 );
  ad = Lunar::getPhase( jDay, Lunar::Q1 );
  printf( "Q1: " );
  printDateTime(ad);
  ***/
  return 0;
}

